import glviskit
import numpy as np

window1 = glviskit.create_window("Window1", 800, 600)

render_list = glviskit.create_render_list()
window1.add_render_list(render_list)

render_list_axes = glviskit.create_render_list()
window1.add_render_list(render_list_axes)

render_list_axes.size(5.0)
render_list_axes.color([1.0, 0.0, 0.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [1.0, 0.0, 0.0])
render_list_axes.color([0.0, 1.0, 0.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [0.0, 1.0, 0.0])
render_list_axes.color([0.0, 0.0, 1.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [0.0, 0.0, 1.0])

render_list.color([1.0, 1.0, 1.0, 1.0])
render_list.size(5.0)
# random points in a cube
render_list.circle(
    np.random.uniform(-1.0, 1.0, size=(100, 3)),
    np.random.uniform(0.0, 1.0, size=(100, 4)),
    np.random.uniform(5.0, 15.0, size=100),
)
# a big surface plane
points = np.array(
    [
        [-1.0, 0.0, -1.0],
        [1.0, 0.0, -1.0],
        [1.0, 0.0, 1.0],
        [-1.0, 0.0, 1.0],
    ]
)
points *= 10.0
points += np.array([0.0, -1.0, 0.0])
render_list.color([0.7, 0.7, 0.7, 1.0])
render_list.fill_polygon(points)
render_list.save()

camera = window1.camera
camera.perspective_fov(60.0, 60.0)
camera.position = [0.0, 0.0, 0.0]
camera.rotation = [0.0, 0.0, 0.0]
camera.preserve_aspect_ratio = True
camera.distance = 5.0

angle = 0.0
frame_index = 0

square = np.array(
    [
        [-1.0, -1.0, 0.0],
        [1.0, -1.0, 0.0],
        [1.0, 1.0, 0.0],
        [-1.0, 1.0, 0.0],
    ]
)
square2 = square + np.array([0.0, 0.0, 1.0])
colors = np.array(
    [
        [1.0, 0.0, 0.0, 1.0],
        [0.0, 1.0, 0.0, 1.0],
        [0.0, 0.0, 1.0, 1.0],
        [1.0, 1.0, 0.0, 1.0],
    ]
)


last_t = glviskit.get_time_seconds()
while glviskit.loop():
    current_t = glviskit.get_time_seconds()
    dt = current_t - last_t

    render_list.restore()
    render_list.color([0.0, 1.0, 0.0, 1.0])
    render_list.size(10.0)
    # path = render_list.path_begin()
    # path2 = render_list.path_begin()
    # for point, color in zip(square, colors):
    #     path.color(color)
    #     path.line_to(point)

    #     path2.color(color)
    #     path2.line_to(point + np.array([0.0, 0.0, 1.0]))
    # path.close()
    # path2.close()
    pts = np.stack([square, square2], axis=0)
    render_list.polygon(pts)
    render_list.circle(pts.reshape(-1, 3))
    render_list.color([1.0, 0.0, 0.0, 0.5])
    render_list.fill_polygon(pts)

    # if dt > 1.0:
    #     last_t = current_t
    #     render_list.enabled = not render_list.enabled



from collections.abc import Sequence
from typing import Annotated, overload

import numpy
from numpy.typing import NDArray

def create_window(
    title: str = "glviskit Window", width: int = 800, height: int = 600
) -> Window: ...
def create_render_list() -> RenderList:
    """Create a new RenderList"""

def get_time_seconds() -> float:
    """Get the current time in seconds since the program started"""

def loop() -> bool:
    """Run the event loop for single iteration and render all windows"""

def render() -> None:
    """Render all windows without processing events"""

class Window:
    def add_render_list(self, rb: RenderList) -> None:
        """Add a RenderList to the window for rendering"""

    @property
    def camera(self) -> Camera:
        """Camera of the window"""

    @camera.setter
    def camera(self, arg: Camera, /) -> None: ...
    @property
    def background_color(self) -> list[float]:
        """Background clear color of the window"""

    @background_color.setter
    def background_color(self, arg: Sequence[float], /) -> None: ...
    @property
    def controller(self) -> BaseController:
        """Camera controller of the window"""

    @controller.setter
    def controller(self, arg: BaseController, /) -> None: ...
    def make_current(self) -> None:
        """Make the window's OpenGL context current"""

    def capture_rgba(
        self,
        image: Annotated[
            NDArray[numpy.uint8], dict(shape=(None, None, 4), order="C", device="cpu")
        ],
    ) -> None:
        """Capture the current window into a preallocated RGBA uint8 array"""

    def render(self) -> None:
        """Render the window's contents"""

class Camera:
    def calculate_transform(
        self,
    ) -> Annotated[NDArray[numpy.float32], dict(shape=(4, 4), order="C")]:
        """Calculate the camera transformation matrix"""

    def perspective_fov(
        self,
        hfov: float,
        vfov: float,
        near: float = 0.10000000149011612,
        far: float = 100.0,
    ) -> None:
        """Set perspective projection using horizontal and vertical FOV"""

    def perspective(
        self,
        fxn: float,
        fyn: float,
        cx: float = 0.5,
        cy: float = 0.5,
        near: float = 0.10000000149011612,
        far: float = 100.0,
    ) -> None:
        """Set perspective projection using focal lengths and principal point"""

    @property
    def position(self) -> list[float]:
        """Position of the camera"""

    @position.setter
    def position(self, arg: Sequence[float], /) -> None: ...
    @property
    def rotation(self) -> list[float]:
        """Rotation of the camera (roll, pitch, yaw in radians)"""

    @rotation.setter
    def rotation(self, arg: Sequence[float], /) -> None: ...
    @property
    def distance(self) -> float:
        """Distance of the camera from its center"""

    @distance.setter
    def distance(self, arg: float, /) -> None: ...
    @property
    def axis_rotation(self) -> list[float]:
        """Axis-convention quaternion rotation (w, x, y, z)"""

    @axis_rotation.setter
    def axis_rotation(self, arg: Sequence[float], /) -> None: ...
    def set_axis_rotation(
        self,
        x: int,
        y: int,
        z: int,
        inv_x: bool = False,
        inv_y: bool = False,
        inv_z: bool = False,
    ) -> None:
        """
        Set axis-convention transform from an axis permutation with optional axis inversions
        """

    @property
    def viewport_size(self) -> list[float]:
        """Viewport size of the camera"""

    @property
    def preserve_aspect_ratio(self) -> bool:
        """Whether to preserve aspect ratio when resizing viewport"""

    @preserve_aspect_ratio.setter
    def preserve_aspect_ratio(self, arg: bool, /) -> None: ...

class RenderList:
    @overload
    def line(
        self,
        starts: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        ends: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple lines from starts to ends"""

    @overload
    def line(
        self,
        starts: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        ends: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple lines from starts to ends"""

    @overload
    def line(self, start: Sequence[float], end: Sequence[float]) -> None:
        """Draw a line from starts to ends"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple points at given positions with given colors and sizes"""

    @overload
    def point(self, point: Sequence[float]) -> None:
        """Draw a point at given position"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw multiple circles at given positions with given colors and sizes"""

    @overload
    def circle(self, point: Sequence[float]) -> None:
        """Draw a circle at given position"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polygons with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a polygon with given vertices"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a polygon with given vertices"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices and colors"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polygon with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
        sizes: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None), device="cpu")
        ],
    ) -> None:
        """Draw multiple polylines with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a polyline with given vertices"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a polyline with given vertices"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices and colors"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def polyline(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """Draw a polyline with given vertices, colors and sizes"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float64], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 3), device="cpu")
        ],
        colors: Annotated[
            NDArray[numpy.float32], dict(shape=(None, None, 4), device="cpu")
        ],
    ) -> None:
        """Draw multiple filled polygons with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a filled polygon with given vertices"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
    ) -> None:
        """Draw a filled polygon with given vertices"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a filled polygon with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a filled polygon with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a filled polygon with given vertices and colors"""

    @overload
    def fill_polygon(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw a filled polygon with given vertices and colors"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices and indices"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices and indices"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices, indices and colors"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices, indices and colors"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices, indices and colors"""

    @overload
    def triangles(
        self,
        vertices: Annotated[
            NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")
        ],
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Draw multiple triangles with given vertices, indices and colors"""

    def path_begin(self) -> Path:
        """
        Create a Path object for drawing complex paths which is save/restore aware
        """

    def mesh_begin(self) -> Mesh:
        """
        Create a Mesh object for incrementally building triangle geometry which is save/restore aware
        """

    def color(self, c: Sequence[float]) -> None:
        """Set the current drawing color"""

    def size(self, size: float) -> None:
        """Set the current drawing size"""

    @overload
    def add_instance(
        self,
        transform: Annotated[NDArray[numpy.float32], dict(shape=(4, 4), device="cpu")],
    ) -> None:
        """Add an instance using a 4x4 transform matrix"""

    @overload
    def add_instance(
        self,
        transform: Annotated[NDArray[numpy.float64], dict(shape=(4, 4), device="cpu")],
    ) -> None:
        """Add an instance using a 4x4 transform matrix"""

    @overload
    def add_instance(
        self,
        pos: Sequence[float],
        rot: Sequence[float],
        scale: Sequence[float] = [1.0, 1.0, 1.0],
    ) -> None:
        """
        Add an instance with position, quaternion rotation (w, x, y, z), and scale
        """

    @overload
    def add_instance(
        self,
        pos: Sequence[float] = [0.0, 0.0, 0.0],
        rot: Sequence[float] = [0.0, 0.0, 0.0],
        scale: Sequence[float] = [1.0, 1.0, 1.0],
    ) -> None:
        """Add an instance with given position, rotation and scale"""

    def save(self) -> None:
        """Save the current render buffer state"""

    def restore(self) -> None:
        """Restore the previously saved render buffer state"""

    def clear(self) -> None:
        """Clear the render buffer"""

    def save_instances(self) -> None:
        """Save the current instances"""

    def restore_instances(self) -> None:
        """Restore the previously saved instances"""

    def clear_instances(self) -> None:
        """Clear the instances"""

    @property
    def enabled(self) -> bool:
        """Whether this RenderList is enabled for rendering"""

    @enabled.setter
    def enabled(self, arg: bool, /) -> None: ...

class Path:
    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively with given colors"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively with given colors"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively with given colors"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> None:
        """Call line_to for multiple points consecutively with given colors"""

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float64], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
        sizes: Annotated[NDArray[numpy.float32], dict(shape=(None,), device="cpu")],
    ) -> None:
        """
        Call line_to for multiple points consecutively with given colors and sizes
        """

    @overload
    def line_to(self, point: Sequence[float]) -> None:
        """Call line_to for a single point"""

    def close(self) -> None:
        """Close the current line sequence"""

    def line_end(self) -> None:
        """End the current line sequence"""

    def color(self, c: Sequence[float]) -> None:
        """Set the current drawing color"""

    def size(self, size: float) -> None:
        """Set the current drawing size"""

class Mesh:
    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
    ) -> list[int]:
        """Add multiple vertices and return their mesh-local indices"""

    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
    ) -> list[int]:
        """Add multiple vertices and return their mesh-local indices"""

    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> list[int]:
        """
        Add multiple vertices with given colors and return their mesh-local indices
        """

    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float64], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> list[int]:
        """
        Add multiple vertices with given colors and return their mesh-local indices
        """

    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float64], dict(shape=(None, 4), device="cpu")],
    ) -> list[int]:
        """
        Add multiple vertices with given colors and return their mesh-local indices
        """

    @overload
    def vertex(
        self,
        points: Annotated[NDArray[numpy.float32], dict(shape=(None, 3), device="cpu")],
        colors: Annotated[NDArray[numpy.float32], dict(shape=(None, 4), device="cpu")],
    ) -> list[int]:
        """
        Add multiple vertices with given colors and return their mesh-local indices
        """

    @overload
    def vertex(self, point: Sequence[float]) -> int:
        """Add a vertex and return its mesh-local index"""

    @overload
    def triangle(
        self,
        indices: Annotated[NDArray[numpy.int32], dict(shape=(None, 3), device="cpu")],
    ) -> None:
        """Add multiple triangles using mesh-local vertex indices"""

    @overload
    def triangle(self, i0: int, i1: int, i2: int) -> None:
        """Add a triangle using mesh-local vertex indices"""

    def color(self, c: Sequence[float]) -> None:
        """Set the current drawing color"""

class BaseController:
    pass

class NullController(BaseController):
    def __init__(self) -> None:
        """Create a NullController"""

class FirstPersonController(BaseController):
    def __init__(self) -> None:
        """Create a FirstPersonController"""

    @property
    def key_move_sensitivity(self) -> float:
        """Sensitivity of movement to key presses"""

    @key_move_sensitivity.setter
    def key_move_sensitivity(self, arg: float, /) -> None: ...
    @property
    def key_rot_sensitivity(self) -> float:
        """Sensitivity of rotation to key presses"""

    @key_rot_sensitivity.setter
    def key_rot_sensitivity(self, arg: float, /) -> None: ...
    @property
    def mouse_sensitivity(self) -> float:
        """Sensitivity of rotation to mouse movement"""

    @mouse_sensitivity.setter
    def mouse_sensitivity(self, arg: float, /) -> None: ...

class SphericalController(BaseController):
    def __init__(self) -> None:
        """Create a SphericalController"""

    @property
    def key_move_sensitivity(self) -> float:
        """Sensitivity of distance change to key presses"""

    @key_move_sensitivity.setter
    def key_move_sensitivity(self, arg: float, /) -> None: ...
    @property
    def key_rot_sensitivity(self) -> float:
        """Sensitivity of rotation to key presses"""

    @key_rot_sensitivity.setter
    def key_rot_sensitivity(self, arg: float, /) -> None: ...
    @property
    def mouse_sensitivity(self) -> float:
        """Sensitivity of rotation to mouse movement"""

    @mouse_sensitivity.setter
    def mouse_sensitivity(self, arg: float, /) -> None: ...
    @property
    def wheel_sensitivity(self) -> float:
        """Sensitivity of distance change to mouse wheel"""

    @wheel_sensitivity.setter
    def wheel_sensitivity(self, arg: float, /) -> None: ...



import glviskit
import numpy as np

window1 = glviskit.create_window("Window1", 800, 600)
window2 = glviskit.create_window("Window2", 800, 600)

render_list = glviskit.create_render_list()
window1.add_render_list(render_list)
window2.add_render_list(render_list)
render_list.clear_instances()

for i in range(1, 5):
    s = 1 if i % 2 == 0 else -1
    render_list.add_instance([3.0 * (i - 0.5), 0, 0], [0.5 * s, 0, 0], [1.0, 1.0, 1.0])
    render_list.add_instance(
        [-3.0 * (i - 0.5), 0, 0], [-0.5 * s, 0, 0], [1.0, 1.0, 1.0]
    )

render_list_sine = glviskit.create_render_list()
window1.add_render_list(render_list_sine)
window2.add_render_list(render_list_sine)

render_list_axes = glviskit.create_render_list()
window1.add_render_list(render_list_axes)
window2.add_render_list(render_list_axes)

render_list_axes.size(5.0)
render_list_axes.color([1.0, 0.0, 0.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [1.0, 0.0, 0.0])
render_list_axes.color([0.0, 1.0, 0.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [0.0, 1.0, 0.0])
render_list_axes.color([0.0, 0.0, 1.0, 1.0])
render_list_axes.line([0.0, 0.0, 0.0], [0.0, 0.0, 1.0])

render_list.color([1.0, 1.0, 1.0, 1.0])
render_list.size(5.0)
render_list.circle(np.random.uniform(-1.0, 1.0, size=(10, 3)))

camera = window1.camera
camera.perspective_fov(60.0, 60.0)
camera.position = [0.0, 0.0, 0.0]
camera.rotation = [0.0, 0.0, 0.0]
camera.preserve_aspect_ratio = True
camera.distance = 15.0

camera2 = window2.camera
camera2.perspective_fov(60.0, 60.0)
camera2.position = [0.0, 0.0, 0.0]
camera2.rotation = [0.0, 0.0, 0.0]
camera2.preserve_aspect_ratio = True
camera2.distance = 15.0

angle = 0.0
frame_index = 0

while glviskit.loop():
    curr_time = glviskit.get_time_seconds()

    frame_index += 1
    angle += 0.005
    camera.rotation = [-0.5, angle, 0.0]
    camera2.rotation = [-0.5, -angle, 0.0]

    render_list.point(
        np.random.uniform(-1.0, 1.0, size=(10, 3)),
        np.random.uniform(0.0, 1.0, size=(10, 4)),
        np.random.uniform(1.0, 2.0, size=10),
    )

    if frame_index % 10 == 0:
        render_list.polyline(
            np.random.uniform(-1.0, 1.0, size=(10, 2, 3)),
            np.random.uniform(0.0, 1.0, size=(1, 2, 4)).repeat(10, axis=0),
            np.random.uniform(1.0, 4.0, size=(1, 2)).repeat(10, axis=0),
        )

    render_list_sine.restore()
    path = render_list_sine.path_begin()

    path.size(4.0)
    ix = np.linspace(-1.0, 1.0, 2000)
    points = np.zeros((2000, 3))
    points[:, 0] = ix * 20.0
    points[:, 1] = 1.5 * np.sin((50.0 * ix) + (10 * curr_time))
    points[:, 2] = 1.5 * np.cos((50.0 * ix) + (10 * curr_time))
    colors = np.zeros((2000, 4))
    colors[:, 0] = (points[:, 0] * 0.5) + 0.5
    colors[:, 1] = (points[:, 1] * 0.5) + 0.5
    colors[:, 2] = 0.5
    colors[:, 3] = 1.0
    path.line_to(points, colors)



import glviskit
import numpy as np

window1 = glviskit.create_window("Window1", 800, 600)

render_list = glviskit.create_render_list()
window1.add_render_list(render_list)

render_list_axes = glviskit.create_render_list()
window1.add_render_list(render_list_axes)

camera = window1.camera
camera.perspective_fov(60.0, 60.0)
camera.position = [0.0, 0.0, 0.0]
camera.rotation = [-0.5, 0.0, 0.0]
# camera.distance = 50.0

window1.controller = glviskit.SphericalController()

xs = np.linspace(-30.0, 30.0, 100)
ys = np.linspace(-30.0, 30.0, 100)

from_p = []
to_p = []
for i in range(len(xs) - 1):
    for j in range(len(ys) - 1):
        from_p.append((xs[i], ys[j], 0.0))
        to_p.append((xs[i + 1], ys[j], 0.0))
        from_p.append((xs[i], ys[j], 0.0))
        to_p.append((xs[i], ys[j + 1], 0.0))


from_p = np.array(from_p, dtype=np.float32)
to_p = np.array(to_p, dtype=np.float32)


t0 = glviskit.get_time_seconds()
while glviskit.loop():
    t = glviskit.get_time_seconds() - t0

    render_list.clear()
    render_list.size(5.0)
    from_p[:, 2] = np.sin(from_p[:, 0] + t) + np.sin(from_p[:, 1] + t)
    to_p[:, 2] = np.sin(to_p[:, 0] + t) + np.sin(to_p[:, 1] + t)
    render_list.line(from_p, to_p)
